package com.supermap.samplecode.data;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Random;

import com.supermap.data.CursorType;
import com.supermap.data.Dataset;
import com.supermap.data.DatasetVector;
import com.supermap.data.Datasets;
import com.supermap.data.Datasource;
import com.supermap.data.DatasourceConnectionInfo;
import com.supermap.data.Datasources;
import com.supermap.data.EngineType;
import com.supermap.data.GeoPoint;
import com.supermap.data.Recordset;
import com.supermap.data.Workspace;

/**
 * <p>
 * Title:记录集管理
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap Objects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：记录集遍历，添加几何对象，修改几何对象，修改属性字段
 * 2、示例数据：安装目录\SampleData\City\Changchun.udb
 * 3、关键类型/成员: 
 *      Recordset.getRecordCount 方法
 *      Recordset.moveFirst 方法
 *      Recordset.moveNext 方法
 *      Recordset.getFieldValue 方法
 *      Recordset.getGeometry 方法
 *      Recordset.addNew 方法
 *      Recordset.update 方法
 *      Recordset.seekID 方法
 *      Recordset.edit 方法
 *      Recordset.setGeometry 方法
 *      Recordset.setFieldValue 方法
 * 4、使用步骤：
 *      (1)输入数字 1 遍历记录集；
 *      (2)输入数字 2 添加几何对象；
 *      (3)输入数字 3 修改几何对象；
 *      (4)输入数字 4 修改属性字段；
 *      (5)输入 q 退出。
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * </p>
 * 
 */

/**
 * <p>
 * Title:Recordset Manager
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------CopyRight----------------------------
 *This file contants the sample code of SuperMap Objects Java.
 * Copyright @2005-2012 SuperMap Software Co., Ltd. All Rights Reserved. 
 * ----------------------------------------------------------------
 * ---------------------SuperMap Objects Java Sample Code Description------------------------
 * 
 * 1. Sample Code Description:Traverse recordset, add geometry object, modify geometry object,modify attribute field.
 * 2. Data: \SampleData\City\Changchun.udb
 * 3. Key Classes/Methods:
 *      Recordset.getRecordCount method
 *      Recordset.moveFirst method
 *      Recordset.moveNext method
 *      Recordset.getFieldValue method
 *      Recordset.getGeometry method
 *      Recordset.addNew method
 *      Recordset.update method
 *      Recordset.seekID method
 *      Recordset.edit method
 *      Recordset.setGeometry method
 *      Recordset.setFieldValue method
 * 4. Procedures:
 * (1)Input 1 to traverse recordset
 * (2)2 to add geometry object
 * (3)3 to  modify Geometry object
 * (4)4 to modify attribute field
 * (5)Input q to quit.
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: SuperMap Software Co., Ltd
 * </p>
 * 
 */

public class SampleRun {

	//所需要的数据
	//Data
	private static String m_hint = "";
	private static String m_print1 = "";
	private static String m_print2 = "";
	private static String m_print3 = "";
	private static String m_print4 = "";
	private static String m_print5 = "";
	private static String m_print6 = "";
	private static String m_print7 = "";
	private static String m_print8 = "";
	private static String m_print9 = "";
	private static String m_print10 = "";
	
	/**
	* 初始化多语言资源
	* Initialize CultureResources
	*/
	private static void InitializeCultureResources()
	{
		if(com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
		{
			m_hint = "\n输入命令如下：\n数字 1 遍历记录集\n数字 2 添加几何对象\n数字 3 修改几何对象\n"
				+ "数字 4 修改属性字段\n输入 q 退出\n";
			m_print1 = "记录集管理示范程序";
			m_print2 = "记录集管理示范程序退出";
			m_print3 = "\n记录集记录数：";
			m_print4 = "遍历记录集开始";
			m_print5 = "记录集中没有记录";
			m_print6 = "遍历记录集结束";
			m_print7 = "添加几何对象完成";
			m_print8 = "修改几何对象完成";
			m_print9 = "记录集中没有几何对象";
			m_print10 = "修改属性字段完成";
		}
		else
		{
			m_hint = "\nInput command as following:\nNumber 1 Traverse recordset\nNumber 2 Add Geometry object\nNumber 3 Modify Geometry object\n"
				+ "Number 4 Modify the attribute field\nInput q Quit\n";
			m_print1 = "Demo for Recordset Manager";
			m_print2 = "Quit Demo for Recordset Manager";
			m_print3 = "\nThe count of the record in the recordset :";
			m_print4 = "Begin the traversing";
			m_print5 = "No record found inthe recordset.";
			m_print6 = "Traverse recordset end";
			m_print7 = "Adding Geometry object completed";
			m_print8 = "Modifying Geometry object completed";
			m_print9 = "No Geometry object found in the recordset";
			m_print10 = "Modifying attribute field complete";
		}
	}
	
	public static void main(String[] args) {

		// 打开工作空间，得到数据源
		// Open the workspace and get the datasource.
		Workspace workspace = new Workspace();
		Datasources datasources = workspace.getDatasources();
		InitializeCultureResources();
		DatasourceConnectionInfo datasourceConnectionInfo = new DatasourceConnectionInfo(
				"../../SampleData/City/Changchun.udb", "recordset", "");
		datasourceConnectionInfo.setEngineType(EngineType.UDB);
		Datasource datasource = datasources.open(datasourceConnectionInfo);

		// 得到数据集集合类，模板方式新建点数据集
		// Get the collection of the dataset, and initialize point dataset by template.
		Datasets datasets = datasource.getDatasets();
		String datasetVectorName = "del_recordset";
		if (datasets.contains(datasetVectorName))
			datasets.delete(datasetVectorName);
		Dataset dataset = datasets.get("BusPoint");
		DatasetVector datasetVector = (DatasetVector) datasets
				.createFromTemplate(datasetVectorName, dataset);

		// 得到数据集的记录集
		// Get the recordset of the dataset.
		Recordset recordset = datasetVector.getRecordset(false,
				CursorType.DYNAMIC);

		BufferedReader bufferedReader = new BufferedReader(
				new InputStreamReader(System.in));
		String hint = m_hint;
		printWithStar(m_print1);
		System.out.println(hint);

		String type = "";
		try {
			while (!type.equals("q")) {
				type = bufferedReader.readLine();
				if (type.equals("1")) {
					listAll(recordset);
				} else if (type.equals("2")) {
					addGeometry(recordset, 20);
				} else if (type.equals("3")) {
					updateGeometry(recordset);
				} else if (type.equals("4")) {
					updateFieldValue(recordset);
				}
				if (!type.equals("q"))
					System.out.println(hint);
			}
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}

		// 释放记录集以及相关资源
		// Dispose the recordset and resources.
		recordset.dispose();
		datasources.closeAll();
		datasourceConnectionInfo.dispose();
		workspace.close();
		workspace.dispose();

		printWithStar(m_print2);

	}

	/**
	 * 遍历记录集，输出记录的ID和Name属性以及几何点对象的坐标
	 * @param recordset 遍历的记录集对象
	 * Traverse recordset, output the ID, name, and the coordinate of the Point object of the record.
	 * @param recordset The Recordset object.
	 */
	private static void listAll(Recordset recordset) {
		int count = recordset.getRecordCount();
		System.out.println(m_print3 + count);
		printWithStar(m_print4);
		if (count == 0) {
			System.out.println(m_print5);
		} else {
			String interval = " ";
			System.out
					.println("SmID         Name                        SmGeometry");
			recordset.moveFirst();
			for (int i = 0; i < recordset.getRecordCount(); i++) {
				if (i > 8)
					interval = "";
				Object valueID = recordset.getFieldValue("SmID");
				if (valueID == null)
					valueID = "";
				Object valueName = recordset.getFieldValue("Name");
				if (valueName == null)
					valueName = "UserName";
				GeoPoint geoPoint = (GeoPoint) recordset.getGeometry();
				System.out.println(valueID + interval + "         " + valueName
						+ "       ( " + geoPoint.getX() + " , "
						+ geoPoint.getY() + " )");
				recordset.moveNext();
			}
		}
		printWithStar(m_print6);
	}

	/**
	 * 向记录集中添加点几何对象
	 * @param recordset 目标记录集
	 * @param count 添加的点几何对象的个数
	 * Add point object to the recordset. 
	 * @param recordset The target recordset.
	 * @param count The count of the added point object.
	 */
	private static void addGeometry(Recordset recordset, int count) {
		for (int i = 0; i < count; i++) {
			double x = 1000 * new Random().nextDouble();
			double y = 800 * new Random().nextDouble();
			GeoPoint geoPoint = new GeoPoint(x, y);
			recordset.addNew(geoPoint);
			recordset.update();
		}
		System.out.println(m_print7);
	}

	/**
	 * 修改ID为id的几何点对象的空间属性
	 * @param recordset 目标记录集
	 * @param id 目标点几何对象的ID
	 * Modify the specified point object's attribute.
	 * @param recordset Target recordset.
	 * @param id Target ID of the Geometry object.
	 */
	private static void updateGeometry(Recordset recordset, int id) {
		recordset.seekID(id);
		GeoPoint geoPoint = new GeoPoint(100, 100);
		recordset.edit();
		recordset.setGeometry(geoPoint);
		recordset.update();
	}

	/**
	 * 修改全部几何点对象的空间属性
	 * @param recordset 目标记录集
	 * Modify all the attribute of the object.
	 * @param recordset Target recordset.
	 */
	private static void updateGeometry(Recordset recordset) {
		int length = recordset.getRecordCount();
		if (length != 0) {
			for (int i = 1; i <= length; i++) {
				updateGeometry(recordset, i);
			}
			System.out.println(m_print8);
		} else {
			System.out.println(m_print9);
		}
	}

	/**
	 * 修改ID为数组中元素的记录的属性字段值
	 * @param recordset 目标记录集
	 * @param id 目标记录的SmID
	 */
	private static void updateFieldValue(Recordset recordset, int id) {
		recordset.seekID(id);
		recordset.edit();
		recordset.setFieldValue("Name", "SuperMap");
		recordset.update();
	}

	/**
	 * 修改记录的全部属性字段值
	 * @param recordset 目标记录集
	 * Modify all the attribute field.
	 * @param recordset Target recordset
	 */
	private static void updateFieldValue(Recordset recordset) {
		int length = recordset.getRecordCount();
		if (length != 0) {
			for (int i = 1; i <= length; i++) {
				updateFieldValue(recordset, i);
			}
			System.out.println(m_print10);
		} else {
			System.out.println(m_print5);
		}
	}

	/**
	 * 以星星的修饰方式输出字符串
	 * @param content 原字符串
	 * Output the string with star crowded.
	 * @param content The string.
	 */
	private static void printWithStar(String content) {
		String star = "****************************";
		content = star + content + star;
		System.out.println(content);
	}

}

